# $Id: getInput.tcl,v 2.5 2003-05-23 16:00:32-04 m1tca00 Exp $
#
# Generic dialog to ask user for entry.
#
# $Author: m1tca00 $

# $Locker:  $
#
# $Log: getInput.tcl,v $
# Revision 2.5  2003-05-23 16:00:32-04  m1tca00
# Handle destruction of window (treat like a "cancel").
#
# Revision 2.4  2000-12-12 10:51:27-05  m1tca00
# Added title and iconname to window.
#
# Revision 2.3  2000/12/04 17:38:34  m1tca00
# Bind Return to OK.
#
# Revision 2.2  2000/11/16 19:30:09  m1tca00
# Release grab after getting input.
#
# Revision 2.1  2000/11/06 19:20:36  m1tca00
# Replaced "Exit" with more accurate "Cancel".
#
# Revision 2.0  2000/11/06 18:38:04  m1tca00
# Replaced with iwidget version.
#
# Revision 1.4  2000/04/07 19:00:00  m1tca00
# Changed font for the prompt.
#
# Revision 1.3  2000/02/25 19:46:02  m1tca00
# Use delete from stooop namespace (delete is a command in Tcl8.2).
#
# Revision 1.2  1998/07/02 20:07:04  m1tca00
# Export promptUser method.
#
# Revision 1.1  1998/07/02 20:04:52  m1tca00
# Initial revision
#

package provide GetInput 2.0

# Required extensions
package require Tclx
package require Iwidgets

# Optional... if available, use the images in it
catch {package require imageLib}

itcl::class GetInput {
    inherit itk::Toplevel

    # State variables
    private variable cancelled 0
    private variable release 0

    # Prompt
    itk_option define -prompt prompt Prompt {} {
	wm title $itk_interior "Input: $itk_option(-prompt)"
	wm iconname $itk_interior "Input: $itk_option(-prompt)"
    }

    # Value
    itk_option define -value value Value {} {}

    # Prompt Font
    itk_option define -promptfont promptfont PromptFont {10x20} {
	catch {$itk_interior.frame.prompt configure \
	    -font $itk_option(-promptfont)}
    }

    # Entry Font
    itk_option define -entryfont entryfont EntryFont {fixed} {
	catch {$itk_interior.frame.entry configure \
	    -font $itk_option(-entryfont)}
    }

    # Entry background color
    itk_option define -entrybg entrybg EntryBg {white} {
	catch {$itk_interior.frame.entry configure \
	    -background $itk_option(-entrybg)}
    }

    # Constructor
    constructor {args} {
	# Hide until ready
	wm withdraw $itk_interior

	# Configure window
	eval itk_initialize -title {{Get Input}} $args

	# Prompt/entry frame
	set f [frame $itk_interior.frame \
		-takefocus 0 \
		-borderwidth 0 \
		-highlightthickness 0]

	# Prompt
	itk_component add prompt {
	    label $f.prompt \
		-takefocus 0 \
		-borderwidth 0 \
		-padx 8 \
		-highlightthickness 0 \
		-textvariable [itcl::scope itk_option(-prompt)]
	}

	# Entry
	itk_component add entry {
	    entry $f.entry \
		-takefocus 1 \
		-borderwidth 4 \
		-relief sunken \
		-width [expr {[string length $itk_option(-value)]+10}] \
		-highlightthickness 0 \
		-foreground blue \
		-background $itk_option(-entrybg) \
		-textvariable [itcl::scope itk_option(-value)]
	}

	# Buttons
	set b [frame $itk_interior.buttons \
		-takefocus 0 \
		-borderwidth 0 \
		-highlightthickness 0]

	# Cancel
	itk_component add cancel {
	    button $b.cancel \
		-text Exit \
		-command [itcl::code $this cancel]
	}
	catch "imageLib::imageConfig $b.cancel Cancel -relief flat"


	# Ok
	itk_component add ok {
	    button $b.ok \
		-text OK \
		-command [itcl::code $this done]
	}
	catch "imageLib::imageConfig $b.ok OK -relief flat"

	# Bind Return to OK
	bind $itk_interior <Return> [itcl::code $this done]

	# Bind destruction to... destruction
	bind $itk_interior <Destroy> \
	    "catch {[itcl::code $this cancel]}"

	# Force reconfiguration of entrybg
	eval configure \
	    -entrybg $itk_option(-entrybg) \
	    -promptfont $itk_option(-promptfont) \
	    -entryfont $itk_option(-entryfont)

	# Pack
	pack $f -side left -fill both -expand 1
	pack $f.prompt -side left -fill x -expand 0
	pack $f.entry -side left -fill x -expand 1
	pack $b -side left -fill both -expand 0 -padx 8
	pack $b.cancel -side top -fill both -expand 0
	pack $b.ok -side top -fill both -expand 0

	# Center the window
	update idletasks
	set x [expr {[winfo screenwidth $itk_interior]/2 - \
		[winfo reqwidth $itk_interior]/2}]
	set y [expr {[winfo screenheight $itk_interior]/2 - \
		[winfo reqheight $itk_interior]/2}]
	wm geometry $itk_interior +$x+$y
    }

    private method done {} {
	set release 1
    }

    private method cancel {} {
	set cancelled 1
	set release 1
    }

    public method activate {} {
	# Initialize state
	set release 0
	set cancelled 0

	# Grab and wait
	wm deiconify $itk_interior
	set oldfocus [focus]
	focus $itk_interior.frame.entry
	grab $itk_interior
	tkwait variable [itcl::scope release]

	# Process response
	catch {
	    grab release $itk_interior
	    focus $oldfocus
	    wm withdraw $itk_interior
	}
	if {$cancelled} {
	    return
	} else {
	    return [list $itk_option(-value)]
	}
    }
}
